package App::Kit::Pangea;

## no critic (RequireUseStrict) - Moo does strict and warnings
use Moo;
extends 'App::Kit';

# in progress

1;

__END__

=encoding utf-8

=head1 App::Kit for Pangea™ 

See L<App::Kit> for details.
