package App::Kit::Obj::CType;

## no critic (RequireUseStrict) - Moo does strict
use Moo;

our $VERSION = '0.1';

has mimeobj => (
    'is'      => 'ro',
    'lazy'    => 1,
    'default' => sub {
        require MIME::Types;
        return MIME::Types->new();
    },
);

sub get_ctype_of_ext {
    my ( $self, $path ) = @_;
    return eval { $self->mimeobj->mimeTypeOf($path)->type() };
}

sub get_ext_of_ctype {
    my ( $self, $ctype ) = @_;
    return eval { wantarray ? $self->mimeobj->type($ctype)->extensions() : ( $self->mimeobj->type($ctype)->extensions() )[0] };
}

1;

__END__

=encoding utf-8

=head1 NAME

App::Kit::Obj::CType - ctype utility object

=head1 VERSION

This document describes App::Kit::Obj::CType version 0.1

=head1 SYNOPSIS

    my $ctype = App::Kit::Obj::CType->new()
    $ctype->get_ctype_of_ext($thing)

=head1 DESCRIPTION

ctype utility object

=head1 INTERFACE

=head2 new()

Returns the object, takes no arguments.

=head2 get_ctype_of_ext(str)

Takes a path (/foo/bar.txt), .ext (.txt), or ext (txt)

Returns the corresponding mime/type (e.g. text/plain).

If it returns nothing and $@ is set there was a problem (described in $@).

=head2 get_ext_of_ctype(ctype)

Takes a mime/type (e.g. text/plain)

In scalar context returns the first extension (e.g. .txt). In list context returns all applicable extensions.

If it returns nothing and $@ is set there was a problem (described in $@).

=head2 mimeobj()

Lazy loaded L<MIME::Types> object.

=head1 DIAGNOSTICS

Throws no warnings or errors of its own.

=head1 CONFIGURATION AND ENVIRONMENT

Requires no configuration files or environment variables.

=head1 DEPENDENCIES

L<MIME::Types>

=head1 INCOMPATIBILITIES

None reported.

=head1 BUGS AND LIMITATIONS

No bugs have been reported.

Please report any bugs or feature requests to
C<bug-app-kit@rt.cpan.org>, or through the web interface at
L<http://rt.cpan.org>.

=head1 AUTHOR

Daniel Muey  C<< <http://drmuey.com/cpan_contact.pl> >>

=head1 LICENCE AND COPYRIGHT

Copyright (c) 2013, Daniel Muey C<< <http://drmuey.com/cpan_contact.pl> >>. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself. See L<perlartistic>.

=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.
