package App::Kit::Util::RW;

## no critic

1;

__END__

=encoding utf-8

=head1 Make the lazy façade methods read/write

This is a boolean module. If it is loaded: true, if it is not: false.

It exists to either be present or be absent. It has no functionality, none, nadda, zilch.

See the mocking part of L<App::Kit> for more information.
